/**
 * @file ksz8851.h
 * @brief KSZ8851 Ethernet controller
 *
 * @section License
 *
 * Copyright (C) 2010-2013 Oryx Embedded. All rights reserved.
 *
 * This file is part of CycloneTCP Open.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * @author Oryx Embedded (www.oryx-embedded.com)
 * @version 1.3.4
 **/

#ifndef _KSZ8851_H
#define _KSZ8851_H

//Enable internal loopback
#define KSZ8851_LOOPBACK_MODE FALSE

//Device ID
#define KSZ8851_REV_A2_ID   0x8870
#define KSZ8851_REV_A3_ID   0x8872

//SPI command set
#define KSZ8851_CMD_RD_REG  0x00
#define KSZ8851_CMD_WR_REG  0x40
#define KSZ8851_CMD_RD_FIFO 0x80
#define KSZ8851_CMD_WR_FIFO 0xC0

//Byte enable bits
#define KSZ8851_CMD_B0      0x04
#define KSZ8851_CMD_B1      0x08
#define KSZ8851_CMD_B2      0x10
#define KSZ8851_CMD_B3      0x20

//KSZ8851 registers
#define KSZ8851_REG_CCR     0x08
#define KSZ8851_REG_MARL    0x10
#define KSZ8851_REG_MARM    0x12
#define KSZ8851_REG_MARH    0x14
#define KSZ8851_REG_OBCR    0x20
#define KSZ8851_REG_EEPCR   0x22
#define KSZ8851_REG_MBIR    0x24
#define KSZ8851_REG_GRR     0x26
#define KSZ8851_REG_WFCR    0x2A
#define KSZ8851_REG_WF0CRC0 0x30
#define KSZ8851_REG_WF0CRC1 0x32
#define KSZ8851_REG_WF0BM0  0x34
#define KSZ8851_REG_WF0BM1  0x36
#define KSZ8851_REG_WF0BM2  0x38
#define KSZ8851_REG_WF0BM3  0x3A
#define KSZ8851_REG_WF1CRC0 0x40
#define KSZ8851_REG_WF1CRC1 0x42
#define KSZ8851_REG_WF1BM0  0x44
#define KSZ8851_REG_WF1BM1  0x46
#define KSZ8851_REG_WF1BM2  0x48
#define KSZ8851_REG_WF1BM3  0x4A
#define KSZ8851_REG_WF2CRC0 0x50
#define KSZ8851_REG_WF2CRC1 0x52
#define KSZ8851_REG_WF2BM0  0x54
#define KSZ8851_REG_WF2BM1  0x56
#define KSZ8851_REG_WF2BM2  0x58
#define KSZ8851_REG_WF2BM3  0x5A
#define KSZ8851_REG_WF3CRC0 0x60
#define KSZ8851_REG_WF3CRC1 0x62
#define KSZ8851_REG_WF3BM0  0x64
#define KSZ8851_REG_WF3BM1  0x66
#define KSZ8851_REG_WF3BM2  0x68
#define KSZ8851_REG_WF3BM3  0x6A
#define KSZ8851_REG_TXCR    0x70
#define KSZ8851_REG_TXSR    0x72
#define KSZ8851_REG_RXCR1   0x74
#define KSZ8851_REG_RXCR2   0x76
#define KSZ8851_REG_TXMIR   0x78
#define KSZ8851_REG_RXFHSR  0x7C
#define KSZ8851_REG_RXFHBCR 0x7E
#define KSZ8851_REG_TXQCR   0x80
#define KSZ8851_REG_RXQCR   0x82
#define KSZ8851_REG_TXFDPR  0x84
#define KSZ8851_REG_RXFDPR  0x86
#define KSZ8851_REG_RXDTTR  0x8C
#define KSZ8851_REG_RXDBCTR 0x8E
#define KSZ8851_REG_IER     0x90
#define KSZ8851_REG_ISR     0x92
#define KSZ8851_REG_RXFCTR  0x9C
#define KSZ8851_REG_TXNTFSR 0x9E
#define KSZ8851_REG_MAHTR0  0xA0
#define KSZ8851_REG_MAHTR1  0xA2
#define KSZ8851_REG_MAHTR2  0xA4
#define KSZ8851_REG_MAHTR3  0xA6
#define KSZ8851_REG_FCLWR   0xB0
#define KSZ8851_REG_FCHWR   0xB2
#define KSZ8851_REG_FCOWR   0xB4
#define KSZ8851_REG_CIDER   0xC0
#define KSZ8851_REG_CGCR    0xC6
#define KSZ8851_REG_IACR    0xC8
#define KSZ8851_REG_IADLR   0xD0
#define KSZ8851_REG_IADHR   0xD2
#define KSZ8851_REG_PMECR   0xD4
#define KSZ8851_REG_GSWUTR  0xD6
#define KSZ8851_REG_PHYRR   0xD8
#define KSZ8851_REG_P1MBCR  0xE4
#define KSZ8851_REG_P1MBSR  0xE6
#define KSZ8851_REG_PHY1ILR 0xE8
#define KSZ8851_REG_PHY1IHR 0xEA
#define KSZ8851_REG_P1ANAR  0xEC
#define KSZ8851_REG_P1ANLPR 0xEE
#define KSZ8851_REG_P1SCLMD 0xF4
#define KSZ8851_REG_P1CR    0xF6
#define KSZ8851_REG_P1SR    0xF8

//CCR register
#define CCR_EEPROM_PRESENCE      (1 << 9)
#define CCR_SPI_MODE             (1 << 8)
#define CCR_32_PIN_PACKAGE       (1 << 0)

//OBCR register
#define OBCR_OUT_DRIVE_STRENGTH  (1 << 6)
#define OBCR_SPI_SO_DELAY2       (1 << 5)
#define OBCR_SPI_SO_DELAY1       (1 << 4)
#define OBCR_SPI_SO_DELAY0       (1 << 3)
#define OBCR_BUS_CLOCK_SEL       (1 << 2)
#define OBCR_BUS_CLOCK_DIV1      (1 << 1)
#define OBCR_BUS_CLOCK_DIV0      (1 << 0)

//EEPCR register
#define EEPCR_EESA               (1 << 4)
#define EEPCR_EESB               (1 << 3)
#define EEPCR_EECB2              (1 << 2)
#define EEPCR_EECB1              (1 << 1)
#define EEPCR_EECB0              (1 << 0)

//MBIR register
#define MBIR_TXMBF               (1 << 12)
#define MBIR_TXMBFA              (1 << 11)
#define MBIR_TXMBFC2             (1 << 10)
#define MBIR_TXMBFC1             (1 << 9)
#define MBIR_TXMBFC0             (1 << 8)
#define MBIR_RXMBF               (1 << 4)
#define MBIR_RXMBFA              (1 << 3)
#define MBIR_RXMBFC2             (1 << 2)
#define MBIR_RXMBFC1             (1 << 1)
#define MBIR_RXMBFC0             (1 << 0)

//GRR register
#define GRR_QMU_MODULE_SOFT_RST  (1 << 1)
#define GRR_GLOBAL_SOFT_RST      (1 << 0)

//WFCR register
#define WFCR_MPRXE               (1 << 7)
#define WFCR_WF3E                (1 << 3)
#define WFCR_WF2E                (1 << 2)
#define WFCR_WF1E                (1 << 1)
#define WFCR_WF0E                (1 << 0)

//TXCR register
#define TXCR_TCGICMP             (1 << 8)
#define TXCR_TCGUDP              (1 << 7)
#define TXCR_TCGTCP              (1 << 6)
#define TXCR_TCGIP               (1 << 5)
#define TXCR_FTXQ                (1 << 4)
#define TXCR_TXFCE               (1 << 3)
#define TXCR_TXPE                (1 << 2)
#define TXCR_TXCE                (1 << 1)
#define TXCR_TXE                 (1 << 0)

//TXSR register
#define TXSR_TXLC                (1 << 13)
#define TXSR_TXMC                (1 << 12)
#define TXSR_TXFID5              (1 << 5)
#define TXSR_TXFID4              (1 << 4)
#define TXSR_TXFID3              (1 << 3)
#define TXSR_TXFID2              (1 << 2)
#define TXSR_TXFID1              (1 << 1)
#define TXSR_TXFID0              (1 << 0)

//RXCR1 register
#define RXCR1_FRXQ               (1 << 15)
#define RXCR1_RXUDPFCC           (1 << 14)
#define RXCR1_RXTCPFCC           (1 << 13)
#define RXCR1_RXIPFCC            (1 << 12)
#define RXCR1_RXPAFMA            (1 << 11)
#define RXCR1_RXFCE              (1 << 10)
#define RXCR1_RXEFE              (1 << 9)
#define RXCR1_RXMAFMA            (1 << 8)
#define RXCR1_RXBE               (1 << 7)
#define RXCR1_RXME               (1 << 6)
#define RXCR1_RXUE               (1 << 5)
#define RXCR1_RXAE               (1 << 4)
#define RXCR1_RXINVF             (1 << 1)
#define RXCR1_RXE                (1 << 0)

//RXCR2 register
#define RXCR2_SRDBL2             (1 << 7)
#define RXCR2_SRDBL1             (1 << 6)
#define RXCR2_SRDBL0             (1 << 5)
#define RXCR2_IUFFP              (1 << 4)
#define RXCR2_RXIUFCEZ           (1 << 3)
#define RXCR2_UDPLFE             (1 << 2)
#define RXCR2_RXICMPFCC          (1 << 1)
#define RXCR2_RXSAF              (1 << 0)

//TXMIR register
#define TXMIR_TXMA_MASK          0x1FFF

//RXFHSR register
#define RXFHSR_RXFV              (1 << 15)
#define RXFHSR_RXICMPFCS         (1 << 13)
#define RXFHSR_RXIPFCS           (1 << 12)
#define RXFHSR_RXTCPFCS          (1 << 11)
#define RXFHSR_RXUDPFCS          (1 << 10)
#define RXFHSR_RXBF              (1 << 7)
#define RXFHSR_RXMF              (1 << 6)
#define RXFHSR_RXUF              (1 << 5)
#define RXFHSR_RXMR              (1 << 4)
#define RXFHSR_RXFT              (1 << 3)
#define RXFHSR_RXFTL             (1 << 2)
#define RXFHSR_RXRF              (1 << 1)
#define RXFHSR_RXCE              (1 << 0)

//RXFHBCR register
#define RXFHBCR_RXBC_MASK        0x0FFF

//TXQCR register
#define TXQCR_AETFE              (1 << 2)
#define TXQCR_TXQMAM             (1 << 1)
#define TXQCR_METFE              (1 << 0)

//RXQCR register
#define RXQCR_RXDTTS             (1 << 12)
#define RXQCR_RXDBCTS            (1 << 11)
#define RXQCR_RXFCTS             (1 << 10)
#define RXQCR_RXIPHTOE           (1 << 9)
#define RXQCR_RXDTTE             (1 << 7)
#define RXQCR_RXDBCTE            (1 << 6)
#define RXQCR_RXFCTE             (1 << 5)
#define RXQCR_ADRFE              (1 << 4)
#define RXQCR_SDA                (1 << 3)
#define RXQCR_RRXEF              (1 << 0)

//TXFDPR register
#define TXFDPR_TXFPAI            (1 << 14)

//RXFDPR register
#define RXFDPR_RXFPAI            (1 << 14)

//IER register
#define IER_LCIE                 (1 << 15)
#define IER_TXIE                 (1 << 14)
#define IER_RXIE                 (1 << 13)
#define IER_RXOIE                (1 << 11)
#define IER_TXPSIE               (1 << 9)
#define IER_RXPSIE               (1 << 8)
#define IER_TXSAIE               (1 << 6)
#define IER_RXWFDIE              (1 << 5)
#define IER_RXMPDIE              (1 << 4)
#define IER_LDIE                 (1 << 3)
#define IER_EDIE                 (1 << 2)
#define IER_SPIBEIE              (1 << 1)
#define IER_DEDIE                (1 << 0)

//ISR register
#define ISR_LCIS                 (1 << 15)
#define ISR_TXIS                 (1 << 14)
#define ISR_RXIS                 (1 << 13)
#define ISR_RXOIS                (1 << 11)
#define ISR_TXPSIS               (1 << 9)
#define ISR_RXPSIS               (1 << 8)
#define ISR_TXSAIS               (1 << 6)
#define ISR_RXWFDIS              (1 << 5)
#define ISR_RXMPDIS              (1 << 4)
#define ISR_LDIS                 (1 << 3)
#define ISR_EDIS                 (1 << 2)
#define ISR_SPIBEIS              (1 << 1)

//CGCR register
#define CGCR_LEDSEL0             (1 << 9)

//IACR register
#define IACR_READ_ENABLE         (1 << 12)
#define IACR_TABLE_SELECT1       (1 << 11)
#define IACR_TABLE_SELECT0       (1 << 10)

//PMECR register
#define PMECR_PME_DELAY_EN       (1 << 14)
#define PMECR_PME_POLARITY       (1 << 12)
#define PMECR_PME_WUP_FRAME_EN   (1 << 11)
#define PMECR_PME_MAGIC_EN       (1 << 10)
#define PMECR_PME_LINK_UP_EN     (1 << 9)
#define PMECR_PME_ENERGY_EN      (1 << 8)
#define PMECR_AUTO_WUP_EN        (1 << 7)
#define PMECR_WUP_NORMAL_OP_MODE (1 << 6)
#define PMECR_WUP_FROM_WUP_FRAME (1 << 5)
#define PMECR_WUP_FROM_MAGIC     (1 << 4)
#define PMECR_WUP_FROM_LINK_UP   (1 << 3)
#define PMECR_WUP_FROM_ENERGY    (1 << 2)
#define PMECR_PWR_MODE1          (1 << 1)
#define PMECR_PWR_MODE0          (1 << 0)

//PHYRR register
#define PHYRR_PHY_RESET          (1 << 0)

//P1MBCR register
#define P1MBCR_LOCAL_LOOPBACK    (1 << 14)
#define P1MBCR_FORCE_100         (1 << 13)
#define P1MBCR_AN_ENABLE         (1 << 12)
#define P1MBCR_RESTART_AN        (1 << 9)
#define P1MBCR_FORCE_FULL_DUPLEX (1 << 8)
#define P1MBCR_HP_MDIX           (1 << 5)
#define P1MBCR_FORCE_MDIX        (1 << 4)
#define P1MBCR_DISABLE_MDIX      (1 << 3)
#define P1MBCR_DISABLE_TRANSMIT  (1 << 1)
#define P1MBCR_DISABLE_LED       (1 << 0)

//P1MBSR register
#define P1MBSR_T4_CAPABLE        (1 << 15)
#define P1MBSR_100_FD_CAPABLE    (1 << 14)
#define P1MBSR_100_CAPABLE       (1 << 13)
#define P1MBSR_10_FD_CAPABLE     (1 << 12)
#define P1MBSR_10_CAPABLE        (1 << 11)
#define P1MBSR_PREAMBLE_SUPPR    (1 << 6)
#define P1MBSR_AN_COMPLETE       (1 << 5)
#define P1MBSR_AN_CAPABLE        (1 << 3)
#define P1MBSR_LINK_STATUS       (1 << 2)
#define P1MBSR_JABBER_TEST       (1 << 1)
#define P1MBSR_EXTENDED_CAPABLE  (1 << 0)

//P1ANAR register
#define P1ANAR_NEXT_PAGE         (1 << 15)
#define P1ANAR_REMOTE_FAULT      (1 << 13)
#define P1ANAR_ADV_PAUSE         (1 << 10)
#define P1ANAR_ADV_100_FD        (1 << 8)
#define P1ANAR_ADV_100           (1 << 7)
#define P1ANAR_ADV_10_FD         (1 << 6)
#define P1ANAR_ADV_10            (1 << 5)
#define P1ANAR_SELECTOR_FIELD4   (1 << 4)
#define P1ANAR_SELECTOR_FIELD3   (1 << 3)
#define P1ANAR_SELECTOR_FIELD2   (1 << 2)
#define P1ANAR_SELECTOR_FIELD1   (1 << 1)
#define P1ANAR_SELECTOR_FIELD0   (1 << 0)

//P1ANLPR register
#define P1ANLPR_NEXT_PAGE        (1 << 15)
#define P1ANLPR_LP_ACK           (1 << 14)
#define P1ANLPR_REMOTE_FAULT     (1 << 13)
#define P1ANLPR_ADV_PAUSE        (1 << 10)
#define P1ANLPR_ADV_100_FD       (1 << 8)
#define P1ANLPR_ADV_100          (1 << 7)
#define P1ANLPR_ADV_10_FD        (1 << 6)
#define P1ANLPR_ADV_10           (1 << 5)

//P1SCLMD register
#define P1SCLMD_VCT_RESULT1      (1 << 14)
#define P1SCLMD_VCT_RESULT0      (1 << 13)
#define P1SCLMD_VCT_EN           (1 << 12)
#define P1SCLMD_FORCE_LNK        (1 << 11)
#define P1SCLMD_REMOTE_LOOPBACK  (1 << 9)

//P1CR register
#define P1CR_LED_OFF             (1 << 15)
#define P1CR_TX_DISABLE          (1 << 14)
#define P1CR_RESTART_AN          (1 << 13)
#define P1CR_DISABLE_AUTO_MDIX   (1 << 10)
#define P1CR_FORCE_MDIX          (1 << 9)
#define P1CR_AN_ENABLE           (1 << 7)
#define P1CR_FORCE_SPEED         (1 << 6)
#define P1CR_FORCE_DUPLEX        (1 << 5)
#define P1CR_ADV_PAUSE           (1 << 4)
#define P1CR_ADV_100_FD          (1 << 3)
#define P1CR_ADV_100             (1 << 2)
#define P1CR_ADV_10_FD           (1 << 1)
#define P1CR_ADV_10              (1 << 0)

//P1SR register
#define P1SR_HP_MDIX             (1 << 15)
#define P1SR_REVERSED_POLARITY   (1 << 13)
#define P1SR_OPERATION_SPEED     (1 << 10)
#define P1SR_OPERATION_DUPLEX    (1 << 9)
#define P1SR_MDIX_STATUS         (1 << 7)
#define P1SR_AN_DONE             (1 << 6)
#define P1SR_LINK_GOOD           (1 << 5)
#define P1SR_PARTNER_ADV_PAUSE   (1 << 4)
#define P1SR_PARTNER_ADV_100_FD  (1 << 3)
#define P1SR_PARTNER_ADV_100     (1 << 2)
#define P1SR_PARTNER_ADV_10_FD   (1 << 1)
#define P1SR_PARTNER_ADV_10      (1 << 0)

//Transmit control word
#define TX_CTRL_TXIC  0x8000
#define TX_CTRL_TXFID 0x003F


/**
 * @brief TX packet header
 **/

typedef __packed struct
{
   uint16_t controlWord;
   uint16_t byteCount;
} Ksz8851TxHeader;


/**
 * @brief RX packet header
 **/

typedef __packed struct
{
   uint16_t statusWord;
   uint16_t byteCount;
} Ksz8851RxHeader;


/**
 * @brief KSZ8851 driver context
 **/

typedef struct
{
   uint_t frameId; ///<Identify a frame and its associated status
} Ksz8851Context;


//KSZ8851 driver
extern const NicDriver ksz8851Driver;

//KSZ8851 related functions
error_t ksz8851Init(NetInterface *interface);

void ksz8851Tick(NetInterface *interface);

void ksz8851EnableIrq(NetInterface *interface);
void ksz8851DisableIrq(NetInterface *interface);

void __irq ksz8851Interrupt(void);
void ksz8851RxEventHandler(NetInterface *interface);

error_t ksz8851SetMacFilter(NetInterface *interface);
error_t ksz8851SendPacket(NetInterface *interface, const uint8_t *data, uint_t length);
uint_t ksz8851ReceivePacket(NetInterface *interface, uint8_t *data, uint_t size);

void ksz8851WriteReg(NetInterface *interface, uint8_t address, uint16_t data);
uint16_t ksz8851ReadReg(NetInterface *interface, uint8_t address);

void ksz8851WriteFifo(NetInterface *interface, const uint8_t *data, uint_t length);
void ksz8851ReadFifo(NetInterface *interface, uint8_t *data, uint_t length);

void ksz8851SetBit(NetInterface *interface, uint8_t address, uint16_t mask);
void ksz8851ClearBit(NetInterface *interface, uint8_t address, uint16_t mask);

uint32_t ksz8851CalcCrc(const void *data, uint_t length);

void ksz8851DumpReg(NetInterface *interface);

#endif
